@@ -49,7 +49,7 @@ class LatticeAdaptor:
         self.table = kwargs.get("table", None)
         self.filename = kwargs.get("file", None)
         self.inputstr = kwargs.get("string", None)
-        self._builder = LatticeBuilderLine()
+        self.builder = LatticeBuilderLine()
 
     @property
     def table(self):
@@ -73,12 +73,17 @@ class LatticeAdaptor:
             lattice format to load from, currenlty lte and madx allowed, by default "lte"
         """
         # use the latticebuilder to build the table
-        self._builder.load_from_file(filename, ftype)
-        self._builder.build_table()
+        self.builder.load_from_file(filename, ftype)
+        self.builder.build_table()
 
-        # extract the relavant info
-        self.name = self._builder.name
-        self.table = self._builder.table
+        # extract the relevant info
+        # Better way?
+        try:
+            self.name = self.builder.name
+        except Exception:
+            self.name = "ring"
+            print("Lattice name has been autoset - check if value is ok.")
+        self.table = self.builder.table
 
         # length is last element center pos + half the  length
         print("Length has been autoset - check if value is ok - otherwise update it.")
@@ -384,6 +389,25 @@ class LatticeAdaptor:
 
         return eq, diff
 
+    def update_table(self):
+        """Reload the table from the builder table."""
+        # updated history
+        self.history.put((deepcopy(self.name), deepcopy(self.len), deepcopy(self.table)))
+
+        self.builder.positions = None
+        self.builder.build_table()
+
+        # extract the relavant info
+        self.name = self.builder.name
+        self.table = self.builder.table
+
+        # length is last element center pos + half the  length
+        print("Length has been autoset - check if value is ok - otherwise update it.")
+        self.len = (
+            self.builder.table.tail(1)["at"].values[-1]
+            + self.builder.table.tail(1)["L"].values[-1] / 2.0
+        )
+
     def undo(self):
         """Undo previous change."""
         if not self.history.empty():
